STATUS_SCHEDULED = 0
STATUS_RUNNING = 1
STATUS_STOPPING = 2
STATUS_STOPPED = 3
STATUS_FAILED = 4
STATUS_SUCCEEDED = 5
STATUS_TIMEOUT = 6
STATUS_SCHEDULED_ERROR = 7

STATUS_DICT = {
    STATUS_SCHEDULED: 'scheduled',
    STATUS_RUNNING: 'running',
    STATUS_STOPPING: 'stopping',
    STATUS_STOPPED: 'stopped',
    STATUS_FAILED: 'failed',
    STATUS_SUCCEEDED: 'succeeded',
    STATUS_TIMEOUT: 'timeout',
    STATUS_SCHEDULED_ERROR: 'scheduled error'
}